@@ -560,6 +560,7 @@ struct probe_point
   };
   std::vector<component*> components;
   const token* tok; // points to first component's functor
+  bool optional;
   void print (std::ostream& o) const;
   probe_point ();
   probe_point(std::vector<component*> const & comps,const token * t);
@@ -577,7 +578,12 @@ struct probe
   probe ();
   void print (std::ostream& o) const;
   virtual void printsig (std::ostream &o) const;
+  virtual probe* basest () { return this; }
   virtual ~probe() {}
+private:
+  static unsigned last_probeidx;
+public:
+  std::string name;
 };
 
 struct probe_alias: public probe
@@ -585,6 +591,7 @@ struct probe_alias: public probe
   probe_alias(std::vector<probe_point*> const & aliases);
   std::vector<probe_point*> alias_names;
   virtual void printsig (std::ostream &o) const;
+  bool epilogue_style;
 };
 
 
